@@ -1,9 +1,9 @@
-#' Fast Imputation of Missing Values by Chained Tree Ensembles
+#' Fast Imputation of Missing Values by Chained Random Forests
 #' 
 #' @importFrom stats var reformulate predict
 #' @importFrom ranger ranger
 #'
-#' @description Uses the "ranger" package [1] to do fast missing value imputation by chained tree ensembles, see [2] and [3].
+#' @description Uses the "ranger" package [1] to do fast missing value imputation by chained random forests, see [2] and [3].
 #' Between the iterative model fitting, it offers the option of predictive mean matching. This firstly avoids imputation
 #' with values not present in the original data (like a value 0.3334 in a 0-1 coded variable). Secondly, predictive mean
 #' matching tries to raise the variance in the resulting conditional distributions to a realistic level and, as such, 
@@ -16,6 +16,7 @@
 #' @param seed Integer seed to initialize the random generator.
 #' @param verbose Controls how much info is printed to screen. 0 to print nothing. 1 (default) to print a "." per iteration and 
 #'                variable, 2 to print the OOB prediction error per iteration and variable (1 minus R-squared for regression).
+#' @param returnOOB Logical flag. If TRUE, the final average out-of-bag prediction error is added to the output as attribute "oob".
 #' @param ... Arguments passed to \code{ranger}. If the data set is large, better use less trees 
 #' (e.g. \code{num.trees = 100}) and/or a low value of \code{sample.fraction}. 
 #' The following arguments are incompatible: \code{formula}, \code{data}, \code{write.forest}, 
@@ -39,9 +40,9 @@
 #' # With extra trees algorithm
 #' irisImputed_et <- missRanger(irisWithNA, pmm.k = 3, num.trees = 100, splitrule = "extratrees")
 #' head(irisImputed_et)
-missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, verbose = 1, ...) {
+missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, verbose = 1, returnOOB = FALSE, ...) {
   if (verbose > 0) {
-    cat("\nMissing value imputation by chained tree ensembles\n")
+    cat("\nMissing value imputation by random forests\n")
   }
   
   stopifnot(is.data.frame(data), dim(data) >= 1L, 
@@ -126,5 +127,12 @@ missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, verbose = 1
   if (verbose > 0) {
     cat("\n")
   }
-  if (j == 2L || (j == maxiter && crit)) data else data.last
+  
+  if (j == 2L || (j == maxiter && crit)) {
+    attr(data, "oob") <- predError
+    return(data)
+  } 
+  
+  attr(data.last, "oob") <- predErrorLast 
+  data.last
 }
